//*****************Cabecera tratamineto.hpp*********************************
//contiene la definicion de una clase que se encarga de computar
//los datos RINEX obtenidos por el receptor GPS.

//Definición de la cabecera para la clase posición y sus miembros
#ifndef procesamientoRinex_HPP
#define procesamientoRinex_HPP


//cebecera basica para de C++ para la entrada y salida
#include<iostream>
#include<vector>
#include<list>
#include<string>
//Libreria que se encarga de leer ficheros y exportalos a texto entendible por los humanos
#include<fstream>
#include<math.h>
#include<wx/string.h>

//cabecera que incluye el manejo de archivos RINEX
#include"rinex.hpp"


//cabecera encargada de extraer datos C1
#include"ExtractC1.hpp"


//cabeceras para el modelo troposférico
#include"TropModel.hpp"
#include"ModeledPR.hpp"
#include"IonoModel.hpp"
#include"IonoModelStore.hpp"
#include"BCEphemerisStore.hpp"
#include"PRSolution.hpp"
#include"icd_200_constants.hpp"


//cabecera encargada de la solucion
#include"SolverWMS.hpp"

//cabecera que se encarga de la solucion del sistema mediante el uso de pesos a los satelites
#include"MOPSWeight.hpp"

//cabecera para la transformacion de deg a rad
#include"geometry.hpp"
 
//Espacio de nombres utilizado
using namespace std;
using namespace gpstk;

//Definición de la clase
class ProcesamientoRinex
{
public:

  //contructores de la clase
  ProcesamientoRinex();
  ~ProcesamientoRinex();

  //metodo de prueba a los archivos
  void testArchivos(Rinex *);

  //metodo de obtencion de la solucion de la posicion a las observacions
  vector<Position> ajusteDatosRinex(Rinex *);

  //Metodo que calcula el vector entre los dos puntos
  double Vector(double & x1, double & x2, double & x3, double & x4, double & x5, double & x6);

  //metodo para obtener las epocas
  vector<double> obtenerEpocasobservacion()
  {return epocasBancroft;}


  //metodos para calcular los promedios de las coordenadas
  double PromedioCoordenadaX();
  double PromedioCoordenadaY();
  double PromedioCoordenadaZ();
  double PromedioLatitud();
  double PromedioLongitud();
  double PromedioAltura();

  //metodos para calcular la varianza
  double VarianzaX();
  double VarianzaY();
  double VarianzaZ();
  double VarianzaLatitud();
  double VarianzaLongitud();
  double VarianzaAltura();

  //metodos para calcular las desviaciones estandar
  double DesviacionEstandarX();
  double DesviacionEstandarY();
  double DesviacionEstandarZ();
  double DesviacionEstandarLatitud();
  double DesviacionEstandarLongitud();
  double DesviacionEstandarAltura();

  void CorreccionesDiferenciales(wxString , wxString , wxString);
  void CrearArchivoReporte(wxString , int &);
  double CorreccionX()
  {return dx;}
  double CorreccionY()
  {return dy;}
  double CorreccionZ()
  {return dz;}

  double CoordenadaCorregidaX();
  double CoordenadaCorregidaY();
  double CoordenadaCorregidaZ();

private:
  //instanciacion de los objetos


  //instancia al objeto Rinex que contiene los archivos 
  //ingresados por el usuario
  Rinex *aptRinex;

  //caracteristicas Rinex
  RinexNavStream archivoNavRinex;
  RinexNavData datoNavRinex;
  RinexNavHeader encabezadoNavRinex;

  RinexMetStream archivoMetRinex;
  RinexMetHeader encabezadoMetRinex;
  RinexMetData datosMetRinex;

  RinexObsStream archivoObsRinex;
  RinexObsHeader ecabezadoObsRinex;
  RinexObsData datoObsRinex;


  //Condiciones metereologicas
  IonoModelStore almacenajeIono;
  BCEphemerisStore almacenajeEfemerides;
  MOPSTropModel modeloTropMops;
  ModeledPR modeloPR;
  ExtractC1 objetoC1;
  IonoModel modeloIonosferico;

  //Validacion datos
  bool posicionInicial;
  Position posicionRealizada;

  //Solucion
  SolverWMS solucionPosicion;
  MOPSWeight solucionPosicionPesos;

  //lista para los segundos de la observacion
  list<RinexObsData> listaSegundos;

  //apuntador del listado de epocas
  list<RinexObsData>::iterator apEpoca;

  //vector donde se almasenaran las posiciones
  vector<Position> coordenadas;

  //apuntador del vector coordendas
  vector<Position>::iterator apCoordenadas;

  //vector de Bancroft aplicado a epocas
  vector<double> epocasBancroft;

  //apuntador a las epocas de Bancroft
  vector<double>::iterator apBancroft;

  //datos estadísticos
  double promedioCoordenadaX, promedioCoordenadaY, promedioCoordenadaZ;
  double promedioLat, promedioLong, promedioAlt;
  double varianzaX, varianzaY, varianzaZ;
  double varianzaLat, varianzaLong, varianzaAlt;
  double desviacionX, desviacionY, desviacionZ;
  double desviacionLat, desviacionLong, desviacionAlt;
  double vectorEntrePuntos;
  double dx, dy, dz;
  double xCorregido, yCorregido, zCorregido;

  string saludo, despedida, avisoArchivo;

};

#endif
