//***********Implementación de las variables miembro de la 
//*************************clase posicion


//llamadas al preprpocesador
#include"procesamientoRinex.hpp"
//#include<fstream>


//constructor de la clase
ProcesamientoRinex::ProcesamientoRinex()
{
  aptRinex = new Rinex;

}

ProcesamientoRinex::~ProcesamientoRinex()
{
  delete aptRinex;
}

void ProcesamientoRinex::testArchivos(Rinex *aptRinex)
{
  //Sentencia para leer los parametros ionosfericos de la observacion
  aptRinex->ObtenerNavegacionesRinex() >> encabezadoNavRinex;

  //Sentencia para adicionar el modelo inosferico desde el archivo cabecera
  modeloIonosferico.setModel(encabezadoNavRinex.ionAlpha, encabezadoNavRinex.ionBeta);

  //solo para la prueba
  cout << "dato iono Alfa: "<< encabezadoNavRinex.ionAlpha[0] << endl;
  cout << "dato iono Beta: "<< encabezadoNavRinex.ionBeta[0] << endl;

  //El mismo modelo se aplicara a toda la observacion
  almacenajeIono.addIonoModel(DayTime::BEGINNING_OF_TIME, modeloIonosferico);

  //Almacenamos las efemerides en el atributo efemeridesGPS
  while(aptRinex->ObtenerNavegacionesRinex() >> datoNavRinex)
    {

      almacenajeEfemerides.addEphemeris(datoNavRinex);
    }
  //Seleccion del mejor criterio para la busqueda de efemerides
  almacenajeEfemerides.SearchPast();

  //obtendra el valor de true si se cuenta con una posicion inicial
  posicionInicial = false;

  cout << fixed << setprecision(8);
  //Configurar la salida para la ipresion mas adelante
}


vector<Position> ProcesamientoRinex::ajusteDatosRinex(Rinex *aptRinex)
{

  while(aptRinex->ObtenerObservacionesRinex() >> datoObsRinex )
    {//Inicio del while
      listaSegundos.push_back(datoObsRinex);
      //Inicio de los datos con suficiente numero de satelites 
      if((datoObsRinex.epochFlag == 0 || datoObsRinex.epochFlag == 1) && (datoObsRinex.numSvs > 3))
	{//inicio de la sentencia if

	  //numero de satelites con datos validos para la epoca
	  int satelitesVal = 0;
	  int resultado;

	  //altura y latitud del receptor para el modelo Troposferico
	  double alturaModeloTropReceptor;
	  double latitudModeloTropReceptor;

	  //Extraemos los datos C1 de las observaciones con información suficiente (4 satelites como minimo)
	  if(objetoC1.getData(datoObsRinex) < 4)
	    {
	      //sentencia que asegura que la siguiente posicion no sea valida
	      posicionInicial = false;
	      continue;

	    }//fin del if para extraer los datos C1

	  if(posicionInicial)
	    {//if que posibilita el uso de una posicion inicial

	      resultado = modeloPR.Prepare(posicionRealizada);
	      
	      //Almacenamos el modelo troposferico
	      alturaModeloTropReceptor = posicionRealizada.getAltitude();
	      latitudModeloTropReceptor = posicionRealizada.getGeodeticLatitude();
	    }
	  else
	    {//Se usa el metodo de Bancroft si no se dispone de una posicion inicial

	      cerr << "Metodo de Bancroft utilizado para esta época" << datoObsRinex.time.DOYsecond() << endl;
	      //	      epocasBancroft.push_back(datoObsRinex.time.DOYsecond());
	      resultado = modeloPR.Prepare(datoObsRinex.time, objetoC1.availableSV, objetoC1.obsData, almacenajeEfemerides);
	      alturaModeloTropReceptor = modeloPR.rxPos.getAltitude();
	      latitudModeloTropReceptor = modeloPR.rxPos.getGeodeticLatitude();

	    }

	  if(resultado)
	    {//En caso de ocurrir algun error con el metodo Prepare(), salta la epoca

	      posicionInicial = false;
	      continue;
	    }//fin del if
	  //En caso de no tener problemas, introducimos el modelo
	  modeloTropMops.setReceiverHeight(alturaModeloTropReceptor);
	  modeloTropMops.setReceiverLatitude(latitudModeloTropReceptor);
	  modeloTropMops.setDayOfYear(datoObsRinex.time.DOY());

	  //Ahora procesamos el modelo GPS para nuestrass observaciones C1
	  satelitesVal = modeloPR.Compute(datoObsRinex.time, objetoC1.availableSV, objetoC1.obsData, almacenajeEfemerides, &modeloTropMops, &almacenajeIono);

	  //Si la informacion cuenta con los datos suficientes los introducimos dentro de mas calculos
	  if(satelitesVal >= 4)
	    {
	      //Procedemos a resolver las ecuaciones de navegacion usando el metodo WMS

	      try
		{//inicio try
		 
		  //Primero coputamos el peso de los satelites
		  int distribucionSatelites = solucionPosicionPesos.getWeights(datoObsRinex.time, modeloPR.availableSV, almacenajeEfemerides, modeloPR.ionoCorrections, modeloPR.elevationSV, modeloPR.azimuthSV, modeloPR.rxPos);

		  //Revicion minima que todo este en orden
		  if( distribucionSatelites != (int)modeloPR.prefitResiduals.size() )
		    {
		      continue;
		    }

		  //Si todo esta en orden resolvemos el sistema
		  solucionPosicion.Compute(modeloPR.prefitResiduals, modeloPR.geoMatrix, solucionPosicionPesos.weightsVector);
		}//fin try

	      catch(InvalidSolver& error)
		{//inicio catch() para la solucion

		  cerr << "No se puede resolver el sistema de ecuaciones para la época " << datoObsRinex.time.DOYsecond() << endl;
		  cerr << error << endl;
		  posicionInicial = false; //la posicion formada no sera valida para la siguiente epoca
		  continue;

		}//fin del catch para la solucion

	      //Con solucionPosicion obtenemos la diferencia vectorial entre una posicion inicial y
	      //una posicion real. Ahora pasaremos ese resultado a un objeto de tipo posicion
	      Position posicionObservador( (modeloPR.rxPos.X() + solucionPosicion.solution[0]), (modeloPR.rxPos.Y() + solucionPosicion.solution[1]), (modeloPR.rxPos.Z() + solucionPosicion.solution[2]) );

	      coordenadas.push_back(posicionObservador);
	      posicionRealizada = posicionObservador;
	      posicionInicial = true;

	    }//fin del if para validar numero de satelites
	  else
	    {
	      posicionInicial = false;//La posicion inicial no sera validada en la siguiente epoca
	    }
	}//fin de la sentencia if para validar numero de satelites
      else
	{
	  //Establece la posicion como falsa para poder calcular una nueva
	  posicionInicial = false;
	}

    }//Fin del while
  //asigna el cominezo del vector a un apuntador iterator
  apCoordenadas = coordenadas.begin();
  //asigna el cominezo del vector a un apuntador iterator
  apBancroft = epocasBancroft.begin();
  //asigna el cominezo del vector a un apuntador iterator
  apEpoca = listaSegundos.begin();  
  //devuelve el vector de tipo Position
  return coordenadas;

}//fin del metodo ajuste




//Promedios para las Coordenadas
double ProcesamientoRinex::PromedioCoordenadaX()
{
  double dato = 0;
  for(int i = 0; i < coordenadas.size(); i++)
    {
      dato += coordenadas[i].X();
    }
  promedioCoordenadaX = dato / coordenadas.size();
  cout << "Coordenada geocéntrica X: " << promedioCoordenadaX << endl;
  return promedioCoordenadaX;
}

double ProcesamientoRinex::PromedioCoordenadaY()
{
  double dato = 0;
  for(int i = 0; i < coordenadas.size(); i++)
    {
      dato += coordenadas[i].Y();
    }
  promedioCoordenadaY = dato / coordenadas.size();
  cout << "Coordenada geocéntrica Y: " << promedioCoordenadaY << endl;
  return promedioCoordenadaY;
}

double ProcesamientoRinex::PromedioCoordenadaZ()
{
  double dato = 0;
  for(int i = 0; i < coordenadas.size(); i++)
    {
      dato += coordenadas[i].Z();
    }
  promedioCoordenadaZ = dato / coordenadas.size();
  cout << "Coordenada geocéntrica Z: " << promedioCoordenadaZ << endl;
  return promedioCoordenadaZ;
}

double ProcesamientoRinex::PromedioLatitud()
{
  double dato = 0;
  for(int i = 0; i < coordenadas.size(); i++)
    {
      dato += coordenadas[i].geodeticLatitude();
    }
  promedioLat = dato / coordenadas.size();
  cout << "Coordenada Latitud: " << promedioLat << endl;
  return promedioLat;
}

double ProcesamientoRinex::PromedioLongitud()
{
  double dato = 0;
  for(int i = 0; i < coordenadas.size(); i++)
    {
      dato += coordenadas[i].longitude();
    }
  promedioLong = dato / coordenadas.size();
  if(promedioLong > 270)
  {
	  promedioLong -= 360;
  }
  cout << "Coordenada Longitud: " << promedioLong << endl;
  return promedioLong;
}

double ProcesamientoRinex::PromedioAltura()
{
  double dato = 0;
  for(int i = 0; i < coordenadas.size(); i++)
    {
      dato += coordenadas[i].height();
    }
  promedioAlt = dato / coordenadas.size();
  cout << "Coordenada Altura: " << promedioAlt << endl;
  return promedioAlt;
}

double ProcesamientoRinex::VarianzaX()
{
  double diferencias = 0, numerador = 0;
  for (int i = 0; i < coordenadas.size(); i++)
    {
      diferencias = coordenadas[i].X() - promedioCoordenadaX;
      diferencias *= diferencias;
      numerador += diferencias; 
    }
      varianzaX = numerador / coordenadas.size();
  return varianzaX;
}

//Varianzas de las coordenadas
double ProcesamientoRinex::VarianzaY()
{
  double diferencias = 0, numerador = 0;
  for (int i = 0; i < coordenadas.size(); i++)
    {
      diferencias = coordenadas[i].Y() - promedioCoordenadaY;
      diferencias *= diferencias;
      numerador += diferencias; 
    }
      varianzaY = numerador / coordenadas.size();
  return varianzaY;
}

double ProcesamientoRinex::VarianzaZ()
{
  double diferencias = 0, numerador = 0;
  for (int i = 0; i < coordenadas.size(); i++)
    {
      diferencias = (coordenadas[i].Z() - promedioCoordenadaZ);
      diferencias *= diferencias;
      numerador += diferencias; 
    }
      varianzaZ = numerador / coordenadas.size();
  return varianzaZ;

}

double ProcesamientoRinex::VarianzaLatitud()
{
  double diferencias = 0, numerador = 0;
  for (int i = 0; i < coordenadas.size(); i++)
    {
      diferencias = coordenadas[i].geodeticLatitude() - promedioLat;
      diferencias *= diferencias;
      numerador += diferencias; 
    }
      varianzaLat = numerador / coordenadas.size();

  return varianzaLat;
}

double ProcesamientoRinex::VarianzaLongitud()
{
  double diferencias = 0, numerador = 0;
  for (int i = 0; i < coordenadas.size(); i++)
    {
      diferencias = coordenadas[i].longitude() - promedioLong;
      diferencias *= diferencias;
      numerador += diferencias; 
    }
      varianzaLong = numerador / coordenadas.size();
  return varianzaLong;
}

double ProcesamientoRinex::VarianzaAltura()
{
  double diferencias = 0, numerador = 0;
  for (int i = 0; i < coordenadas.size(); i++)
    {
      diferencias = coordenadas[i].height() - promedioAlt;
      diferencias *= diferencias;
      numerador += diferencias; 
    }
      varianzaAlt = numerador / coordenadas.size();
  return varianzaAlt;
}


//Desciaciones estandar de las coordenadas
double ProcesamientoRinex::DesviacionEstandarX()
{
  desviacionX = sqrt(varianzaX);
  return desviacionX;
}

double ProcesamientoRinex::DesviacionEstandarY()
{
  desviacionY = sqrt(varianzaY);
  return desviacionY;
}

double ProcesamientoRinex::DesviacionEstandarZ()
{
  desviacionZ = sqrt(varianzaZ);
  return desviacionZ;
}

double ProcesamientoRinex::DesviacionEstandarLatitud()
{
  desviacionLat = sqrt(varianzaLat);
  return desviacionLat;
}

double ProcesamientoRinex::DesviacionEstandarLongitud()
{
  desviacionLong = sqrt(varianzaLong);
  return desviacionLong;
}

double ProcesamientoRinex::DesviacionEstandarAltura()
{
  desviacionAlt = sqrt(varianzaAlt);
  return desviacionAlt;
}

void ProcesamientoRinex::CorreccionesDiferenciales(wxString x, wxString y, wxString z)
{
  double datoX, datoY, datoZ;
  //el valor x,y,z; debe llevar como separador decimal la ","
  x.ToDouble(&datoX);
  y.ToDouble(&datoY);
  z.ToDouble(&datoZ);

  cout << "datoX: " << datoX << endl;
  cout << "datoY: " << datoY << endl;
  cout << "datoZ: " << datoZ << endl;

  dx = promedioCoordenadaX - datoX;
  dy = promedioCoordenadaY - datoY;
  dz = promedioCoordenadaZ - datoZ;

  cout << "dx: " << dx << endl;
  cout << "dy: " << dy << endl;
  cout << "dz: " << dz << endl;

}

double ProcesamientoRinex::CoordenadaCorregidaX()
{
  //cout << "Valor dx: " << dx <<endl;
  xCorregido = promedioCoordenadaX + dx;
  cout << "nuevo x: " << fixed << setprecision(8) <<  xCorregido << endl;
  return xCorregido;
}

double ProcesamientoRinex::CoordenadaCorregidaY()
{
  yCorregido = promedioCoordenadaY + dy;
  cout << "nuevo y: " << yCorregido << endl;
  return yCorregido;
}

double ProcesamientoRinex::CoordenadaCorregidaZ()
{
  zCorregido = promedioCoordenadaZ + dz;
  cout << "nuevo z: " << zCorregido << endl;
  return zCorregido;
}


double ProcesamientoRinex::Vector(double & x1, double & x2, double & y1, double & y2, double & z1, double & z2)
{
  double deltaX, deltaY, deltaZ;
  deltaX = x2 - x1;
  deltaY = y2 - y1;
  deltaZ = z2 - z1;

  deltaX *= deltaX;
  deltaY *= deltaY;
  deltaZ *= deltaZ;

  vectorEntrePuntos = sqrt((deltaX + deltaY) + deltaZ);

  return vectorEntrePuntos;
}




void ProcesamientoRinex::CrearArchivoReporte(wxString n, int & tipo)
{
  ofstream archivoEscritura;


  if(tipo == 1)
    {
      
      archivoEscritura.open(n, ios::out|ios::trunc);
      archivoEscritura << fixed << setprecision(8);
      archivoEscritura << "Bienvenido a GUDX\n";
      archivoEscritura << "Programa para post-procesamiento de datos GPS v1\n";
      archivoEscritura << "Coordenas de la Base por época:\n";
      archivoEscritura << "\n";
      archivoEscritura << "\n";
      archivoEscritura << "     X(m) " << "               ";
      archivoEscritura << "Y(m)  " << "           ";
      archivoEscritura << "  Z(m) " << "            ";
      archivoEscritura << " Latitud " << "      ";
      archivoEscritura << " Longitud" << "      ";
      archivoEscritura << " Altura" <<endl;
      for(int i = 0; i < coordenadas.size(); i++)
	{
	  archivoEscritura << coordenadas[i].X() << "   ";              //coordenada X geocentrica
	  archivoEscritura << coordenadas[i].Y() << "   ";              //coordenada y geocentrica
	  archivoEscritura << coordenadas[i].Z() << "   ";              //coordenada Z geocentrica
	  archivoEscritura << coordenadas[i].geodeticLatitude() << "   ";//Latitud
	  archivoEscritura << coordenadas[i].longitude() - 360 << "   ";//Longitud para Colombia 
	  archivoEscritura << coordenadas[i].height() << "   " << endl;  //Altura elipsoidal
	  
	}
      archivoEscritura << "\n";
      archivoEscritura << "epocas: " << coordenadas.size() << endl;
      archivoEscritura << "\n";
      archivoEscritura << "Promedios de las coordenadas geocéntricas:\n";
      archivoEscritura << "X: " << promedioCoordenadaX << endl;
      archivoEscritura << "Y: " << promedioCoordenadaY << endl;
      archivoEscritura << "Z: " << promedioCoordenadaZ << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << "Promedios de las coordenadas geográficas:\n";
      archivoEscritura << "Latitud: " << promedioLat << endl;
      archivoEscritura << "Longitud: " << promedioLong << endl;
      archivoEscritura << "Altura: " << promedioAlt << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << fixed << setprecision(15);
      archivoEscritura << "Varianzas de las coordenadas geocéntricas:\n";
      archivoEscritura << "Varianza en X: " << varianzaX << endl;
      archivoEscritura << "Varianza en Y: " << varianzaY << endl;
      archivoEscritura << "Varianza en Z: " << varianzaZ << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << "Varianza de las coordenadas geográficas:\n";
      archivoEscritura << "Varianza en la Latitud: "  << scientific << varianzaLat << endl;
      archivoEscritura << "Varianza en la Longitud: " << varianzaLong << endl;
      archivoEscritura << "Varianza en la Altura: " << varianzaAlt << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << "Desviación estándar de las geocéntricas:\n";
      archivoEscritura << "Desviación en X: " << desviacionX << endl;
      archivoEscritura << "Desviación en Y: " << desviacionY << endl;
      archivoEscritura << "Desviacion en Z: " << desviacionZ << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << "Desviación estándar de las geográficas:\n";
      archivoEscritura << "Desviación en la Latitud: " << desviacionLat << endl;
      archivoEscritura << "Desviación en la longitud: " << desviacionLong << endl;
      archivoEscritura << "Desviacion en la Altura: " << desviacionAlt << endl;
      archivoEscritura << "\n";

      archivoEscritura << "Correcciones a las Coordenadas: " << endl;
       archivoEscritura << fixed << setprecision(15);     
      archivoEscritura << "dx: " << dx << endl;
      archivoEscritura << "dy: " << dy << endl; 
      archivoEscritura << "dz: " << dz << endl;
      archivoEscritura << "\n";
      archivoEscritura << "Coordenadas Corregidas:\n";
      archivoEscritura << "X: " << xCorregido << endl;
      archivoEscritura << "Y: " << yCorregido << endl;
      archivoEscritura << "Z: " << zCorregido << endl;
      archivoEscritura << "\n";


      archivoEscritura << "Siguientes datos...\n";
      archivoEscritura.close();
      coordenadas.clear();
    }
  if(tipo == 2)
    {
   
      cout << "numero de registro antes de comenzar el rover: " << coordenadas.size() << endl;
      archivoEscritura.open(n, ios::out|ios::app);
      archivoEscritura << fixed << setprecision(8);
      archivoEscritura << "\n";
      archivoEscritura << "Coordenas del Rover por época:\n";
      archivoEscritura << "\n";
      archivoEscritura << "\n";
      archivoEscritura << "     X(m) " << "               ";
      archivoEscritura << "Y(m)  " << "           ";
      archivoEscritura << "  Z(m) " << "            ";
      archivoEscritura << " Latitud " << "      ";
      archivoEscritura << " Longitud" << "      ";
      archivoEscritura << " Altura" <<endl;
      for(int i = 0; i < coordenadas.size(); i++)
	{
	  archivoEscritura << coordenadas[i].X() << "   ";              //coordenada X geocentrica
	  archivoEscritura << coordenadas[i].Y() << "   ";              //coordenada y geocentrica
	  archivoEscritura << coordenadas[i].Z() << "   ";              //coordenada Z geocentrica
	  archivoEscritura << coordenadas[i].geodeticLatitude() << "   ";//Latitud
	  archivoEscritura << coordenadas[i].longitude() - 360 << "   ";//Longitud para Colombia 
	  archivoEscritura << coordenadas[i].height() << "   " << endl;  //Altura elipsoidal
	  
	}
      archivoEscritura << "\n";
      archivoEscritura << "Número de regsitros: " << coordenadas.size() << endl;
      archivoEscritura << "\n";
      archivoEscritura << "Promedios de las coordenadas geocéntricas:\n";
      archivoEscritura << "X: " << promedioCoordenadaX << endl;
      archivoEscritura << "Y: " << promedioCoordenadaY << endl;
      archivoEscritura << "Z: " << promedioCoordenadaZ << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << "Promedios de las coordenadas geográficas:\n";
      archivoEscritura << "Latitud: " << promedioLat << endl;
      archivoEscritura << "Longitud: " << promedioLong << endl;
      archivoEscritura << "Altura: " << promedioAlt << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << fixed << setprecision(15);
      archivoEscritura << "Varianzas de las coordenadas geocéntricas:\n";
      archivoEscritura << "Varianza en X: " << varianzaX << endl;
      archivoEscritura << "Varianza en Y: " << varianzaY << endl;
      archivoEscritura << "Varianza en Z: " << varianzaZ << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << "Varianza de las coordenadas geográficas:\n";
      archivoEscritura << "Varianza en la Latitud: "  << scientific << varianzaLat << endl;
      archivoEscritura << "Varianza en la Longitud: " << varianzaLong << endl;
      archivoEscritura << "Varianza en la Altura: " << varianzaAlt << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << "Desviación estándar de las geocéntricas:\n";
      archivoEscritura << "Desviación en X: " << desviacionX << endl;
      archivoEscritura << "Desviación en Y: " << desviacionY << endl;
      archivoEscritura << "Desviacion en Z: " << desviacionZ << endl;
      archivoEscritura << "\n";
      
      archivoEscritura << "Desviación estándar de las geográficas:\n";
      archivoEscritura << "Desviación en la Latitud: " << desviacionLat << endl;
      archivoEscritura << "Desviación en la longitud: " << desviacionLong << endl;
      archivoEscritura << "Desviacion en la Altura: " << desviacionAlt << endl;
      archivoEscritura << "\n";

      archivoEscritura << fixed << setprecision(15);
      archivoEscritura << "Correcciones a las Coordenadas: " << endl;
      archivoEscritura << "dx: " << dx << endl;
      archivoEscritura << "dy: " << dy << endl; 
      archivoEscritura << "dz: " << dz << endl;
      archivoEscritura << "\n";

      archivoEscritura << "Coordenadas Corregidas:\n";
      archivoEscritura << "X: " << xCorregido << endl;
      archivoEscritura << "Y: " << yCorregido << endl;
      archivoEscritura << "Z: " << zCorregido << endl;
      archivoEscritura << "\n";
      archivoEscritura << "\n";
      archivoEscritura << "\n";

      archivoEscritura << "El vector calculado entre los dos puntos es:\n";
      archivoEscritura << "Magnitud: " << fixed << setprecision(10) << vectorEntrePuntos << " Mts.\n";
      archivoEscritura.close();
      coordenadas.clear();      
      }

}



